##############################################################################
#
# Copyright (c) 2002 Ingeniweb SARL
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Plone Article migration module

$Id: Migrator.py,v 1.2 2004/12/28 10:38:37 pjgrizel Exp $
"""

__author__ = ''
__docformat__ = 'restructuredtext'

# Python imports
from StringIO import StringIO

# Zope imports
from OFS.Image import File

# Archetypes imports
from Products.Archetypes.interfaces.base import IBaseUnit

# CMF imports
from Products.CMFCore.utils import getToolByName


class ZAttachmentAttribute:
    """Fake ZAttachmentAttribute class. Used to get attachment"""
    __underlyingFile__ = None
    title = ''
    filename = ''
    content_type = ''
    
class ZDummyAttachment:
    """Fake ZDummyAttachment class. Used to get attachment data"""
    content = ''

class Migrator:
    """Migration class
    """
    
    def migrate20To30(self, portal, article_brains, stdout, options):
        """Migrate PloneArticle 2.x to PloneArticle 3.x
        """
        
        portal_article = getToolByName(portal, 'portal_article')
        image_type = portal_article.getAllowedImageType()
        attachment_type = portal_article.getAllowedAttachmentType()
        dry_run = options.get('dry_run', False)
        external_images = options.get('external_images', False)
        external_files = options.get('external_files', False)
        
        if stdout is not None:
            stdout.write('Migrate PloneArticle v2.x to v3.x.\r\n')
        
        nb_articles = 0
        
        # Get all PloneArticle in v2.x
        for article_brain in article_brains:
            article = article_brain.getObject()
            
            if hasattr(article, 'images_ids'):
                # This is an article from v2.x
                parent = article.getParentNode()
                nb_articles += 1
                
                if not dry_run:
                    image_ids = article.images_ids
                    attachment_ids = article.attachments_ids
                
                    # Migrate images
                    for image_id in image_ids:
                        image = getattr(article, image_id)
                        title = image.title_or_id()
                        setattr(image, 'filename', image.getId())
                        
                        if external_images:
                            new_id = portal.generateUniqueId()
                            parent.invokeFactory(image_type, new_id, title=title, image=image)
                            new_obj = getattr(parent, new_id)
                            article.addImageFromObject(new_obj)
                        else:
                            article.addImage(title=title, description='', image=image)
                        
                    # Migrate attachments
                    for attachment_id in attachment_ids:
                        attachment = getattr(article, attachment_id)
                        attachment.__module__ = 'ZAttachmentAttribute'
                        content_type = attachment.content_type
                        attachment_file = attachment.__underlyingFile__
                        attachment_file.__module__ = 'ZDummyAttachment'
                        title = attachment.title
                        data = attachment_file.content
                        new_file = File('new_file', title, data, content_type)
                        setattr(new_file, 'filename', attachment.filename)
                        
                        if external_images:
                            new_id = portal.generateUniqueId()
                            parent.invokeFactory(attachment_type, new_id, title=title, file=new_file)
                            new_obj = getattr(parent, new_id)
                            article.addAttachmentFromObject(new_obj)
                        else:
                            article.addAttachment(title=title, description='', file=new_file)
                    
                    # Migrate models
                    root_model_name = 'PloneArticleModel'
                    model_nb = article.model[len(root_model_name):]
                    article.model = 'plonearticle_model%s' % model_nb
                    
                    # Delete old attributes
                    article.manage_delObjects(image_ids)
                    article.manage_delObjects(attachment_ids)
                    del article.images_ids
                    del article.attachments_ids
            
        if stdout is not None:
            stdout.write('%s article(s) migrated.\r\n' % nb_articles)
            stdout.write('Migration from PloneArticle v2.x to v3.x finished.\r\n')
            
    def migrate(self, portal, stdout=None, options={}):
        cat_tool = getToolByName(portal, 'portal_catalog')
        article_brains = cat_tool(portal_type='PloneArticle')
        self.migrate20To30(portal, article_brains, stdout, options)



# Others migration script

# Migrate PA created with AT 1.3
# Mimetype of text is text/plain. Change it to text/html

def change_text_mimetype(self, mimetype='text/html', portal_types=None):
    """Migrate mimetype of text_field"""
    
    out = StringIO()
    cat_tool = getToolByName(self, 'portal_catalog')
    
    if portal_types is None:
        portal_types = 'PloneArticle'
    
    article_brains = cat_tool(portal_type=portal_types)
    
    for article_brain in article_brains:
        article = article_brain.getObject()
        
        # Get base unit 
        text_field = article.getField('text')
        bu = text_field.getBaseUnit(article)
        if IBaseUnit.isImplementedBy(bu):
            bu.mimetype = mimetype
            out.write("%s (%s) converted.\n" % (article.absolute_url_path(), article.portal_type))
        
    out.write('Mimetypes changed.\n')
    return out.getvalue()
